﻿//#define WEIGHT

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using LC;

namespace BthDemo
{
    public delegate void OnGetText(string str);

    public partial class Form1 : Form
    {
        Bluetooth bth = new Bluetooth();
        List<DeviceInfo> deviceList = new List<DeviceInfo>();

        bool bNeedPowerOff;

        string pairedDevice;

        byte[] buffer = new byte[1024];

#if WEIGHT
        int bufferLen = 0;

        byte[] lastValue = new byte[16];
        
        [DllImport("coredll", SetLastError = true)]
        internal static unsafe extern int memcmp(byte* buf1, byte* buf2, uint count);

        private int BlockCompare(byte[] buf1, int index1, byte[] buf2, int index2, uint count)
        {
            unsafe {
                fixed(byte* p1 = buf1, p2 = buf2)
                {
                    return memcmp(p1 + index1, p2 + index2, count);
                }
            }
        }

#endif

        public Form1()
        {
            InitializeComponent();
            bth.Port.DataReceived += new System.IO.Ports.SerialDataReceivedEventHandler(port_DataReceived);
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            if (Device.GetBthPowerStatus() == 0)
            {
                Device.EnableBthModule();
                bNeedPowerOff = true;
            }
        }

        private void Form1_Closed(object sender, EventArgs e)
        {
            if (bth.IsConnected)
            {
               bth.Disconnect();
            }

            if (bNeedPowerOff)
            {
                Device.DisableBthModule();
            }
        }

        private void btnSearch_Click(object sender, EventArgs e)
        {
            using (Form2 frm = new Form2(bth, deviceList))
            {
                frm.ShowDialog();
                this.addrList.Items.Clear();
                foreach (DeviceInfo info in deviceList)
                {
                    this.addrList.Items.Add(info.Name);
                }
            }
        }

        private void btnConn_Click(object sender, EventArgs e)
        {
            if (addrList.Items.Count == 0 || addrList.SelectedIndex == -1)
            {
                return;
            }

            if (!bth.IsConnected)
            {
                string addr = "";

                string name = (string)addrList.SelectedItem;
                foreach (DeviceInfo info in deviceList)
                {
                    if (info.Name == name)
                    {
                        addr = info.Address;
                        break;
                    }
                }

                if (bth.Pair(addr, pin.Text))
                {
                    pairedDevice = addr;
                    if (bth.Connect(addr))
                    {
                        bthConn.Enabled = false;
                        bthDisconn.Enabled = true;
                        bthSend.Enabled = true;
                    }
                }
            }
        }

        private void btnDisconn_Click(object sender, EventArgs e)
        {
            if (bth.IsConnected)
            {
                bth.Disconnect();
                bth.UnPair(pairedDevice);
                pairedDevice = null;

                bthConn.Enabled = true;
                bthDisconn.Enabled = false;
                bthSend.Enabled = false;
            }
        }

        private void btnSend_Click(object sender, EventArgs e)
        {
            if (bth.IsConnected)
            {
                byte[] bytes = Encoding.ASCII.GetBytes(send.Text);

                bth.Port.Write(bytes, 0, bytes.Length);
            }
        }


        void port_DataReceived(object sender, System.IO.Ports.SerialDataReceivedEventArgs e)
        {

#if !WEIGHT
            int len =bth.Port.BytesToRead;
            bth.Port.Read(buffer, 0, len);

            this.BeginInvoke((OnGetText)delegate(String str)
            {
                if (recv.Text.Length >= 128)
                {
                    recv.Text = "";
                }
                recv.Text += str;

            }, Encoding.ASCII.GetString(buffer, 0, len));

#else

            int bytesToRead = bth.port.BytesToRead;
            if (bytesToRead > buffer.Length - bufferLen)
            {
                bufferLen = 0;
            }

            int bytesRead = bth.port.Read(buffer, bufferLen, bytesToRead);
            if (bytesRead <= 0)
            {
                return;
            }
            bufferLen += bytesRead;


            //解析收到的数据
            int startIndex = 0;
            while (startIndex < bufferLen)
            {
                int endIndex = Array.IndexOf(buffer, (byte)'=', startIndex, bufferLen - startIndex);
                if (endIndex != -1)
                {
                    int len = endIndex - startIndex;

                    if (BlockCompare(lastValue, 0, buffer, startIndex, (uint)len) != 0)
                    {
                        Buffer.BlockCopy(buffer, startIndex, lastValue, 0, len);

                        Array.Reverse(buffer, startIndex, len);


                        this.BeginInvoke((OnGetText)delegate(String str)
                        {
                            textBox2.Text = str;

                        }, Encoding.ASCII.GetString(buffer, startIndex, len));
                    }

                    startIndex = endIndex + 1;
                }
                else
                {
                    break;
                }
            }

            bufferLen -= startIndex;
            Buffer.BlockCopy(buffer, startIndex, buffer, 0, bufferLen);

#endif


        }

    }
}