﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using LC;

namespace GPSDemo
{
    public partial class Form1 : Form
    {
        const string CRLF = "\r\n";
        GpsManager gps = new GpsManager();
        int statusCount = 0;

        public Form1()
        {
            InitializeComponent();
            this.gps.OnGetGPSInfo += new GetGPSInfoHandler(GetGPSInfo);
            this.btnStart.Enabled = true;
            this.btnStop.Enabled = false;
        }

        private void btnStart_Click(object sender, EventArgs e)
        {
            statusCount = 0;
            if (gps.Start())
            {
                this.textBox1.SelectedText = "GPS Module has started!" + CRLF;
                this.textBox1.ScrollToCaret();
                this.btnStop.Enabled = true;
                this.btnStart.Enabled = false;
            }
            else
            {
                this.textBox1.SelectedText = "GPS Module started fail!" + CRLF;
                this.textBox1.ScrollToCaret();
            }
        }

        private void btnStop_Click(object sender, EventArgs e)
        {
            gps.Stop();
            this.textBox1.SelectedText = "\r\nGPS Module has closed!" + CRLF;
            this.textBox1.ScrollToCaret();
            this.btnStop.Enabled = false;
            this.btnStart.Enabled = true;
        }

        private void GetGPSInfo(object sender, GPSEventArgs e)
        {
            if (this.InvokeRequired)
            {
                this.Invoke((GetGPSInfoHandler)delegate
                {
                    OnGetGPS(sender, e);
                }, new object[] { sender, e });
            }
            else
            {
                OnGetGPS(sender, e);
            }
        }

        private void OnGetGPS(object sender, GPSEventArgs e)
        {
            this.txtStatus.Text = e.Status ? "Located" : "Lost";
            this.txtSalCount.Text = e.SatelliteTotalNums.ToString();
            this.txtlongitude.Text = String.Format("{0} {1}°", (e.LongitudeDegree > 0 ? "E" : "W"), (e.LongitudeDegree + e.LongitudeMinute / 60));
            this.txtlatitude.Text = String.Format("{0} {1}°", (e.LatitudeDegree > 0 ? "N" : "S"), (e.LatitudeDegree + e.LatitudeMinute / 60));

            if ((++statusCount % 1000) == 0)
            {
                statusCount = 0;
                this.txtStatus.Text = String.Empty;
            }

            this.textBox1.SelectedText = String.Format("\r\nLocation Status:{0}\r\nNumber of satellites:{1}\r\n********************", 
                this.txtStatus.Text, this.txtSalCount.Text);
            this.textBox1.ScrollToCaret();
        }

        private void Form1_Closed(object sender, EventArgs e)
        {
            if (gps.IsStart)
            {
                gps.Stop();
            }
        }
    }
}