﻿using System;
using System.Runtime.InteropServices;
using System.Collections.Generic;
using System.Text;
using LC;
using System.Threading;

namespace TestSms
{


    public class SmsReceivedEventArgs : EventArgs
    {
        private int _index;
        public int Index
        {
            get { return _index; }
            set { _index = value; }
        }
        public SmsReceivedEventArgs(int index)
        {
            _index = index;
        }
    }

    public class SmsSendResultEventArgs : EventArgs
    {
        private bool _result;
        public bool Result
        {
            get { return _result; }
            set { _result = value; }
        }

        public SmsSendResultEventArgs(bool result)
        {
            _result = result;
        }
    }


    public class Sms
    {
        public Sms(string phoneNum, string content)
        {
            this.phoneNum = phoneNum;
            this.content = content;
        }

        public Sms(string phoneNum, string content, string time)
        {
            this.phoneNum = phoneNum;
            this.content = content;
            this.time = time;
        }

        string phoneNum;
        public string PhoneNum
        {
            get { return phoneNum; }
            set { phoneNum = value; }
        }

        string content;
        public string Content
        {
            get { return content; }
            set { content = value; }
        }

        string time;
        public string Time
        {
            get { return time; }
            set { time = value; }
        }
    } 


    class SmsMgr
    {
        public event EventHandler<SmsReceivedEventArgs> SmsReceived;
        public event EventHandler<SmsSendResultEventArgs> SmsSent;

        private bool needClose = false;



        private IntPtr[] hRecvEvent = new IntPtr[2];
        private IntPtr hNotify = IntPtr.Zero;
        private Thread recvThread;



        private Queue<Sms> smsQueue;
        private IntPtr[] hSendEvent = new IntPtr[2];
        private Thread sendThread;



        private bool isOpened = false;

        public bool IsOpened
        {
            get { return isOpened; }
            set { isOpened = value; }
        }


        public bool Open()
        {

            if (!Device.SMS_IsOpened())
            {
                if (!Device.SMS_Open())
                {
                    return false;
                }
                needClose = true;
            }

            Win32.MSGQUEUEOPTIONS options = new Win32.MSGQUEUEOPTIONS();
            options.dwSize = 20;
            options.dwFlags = 2;
            options.dwMaxMessages = 64;
            options.cbMaxMessage = 2;
            options.bReadAccess = true;

            IntPtr hMsgQ = Win32.CreateMsgQueue(null, options);
            if (hMsgQ == IntPtr.Zero)
            {
                return false;
            }

            hNotify = Device.SMS_RegisterNotification(hMsgQ);
            if (hNotify == IntPtr.Zero)
            {
                return false;
            }

            hRecvEvent[0] = Win32.CreateEvent(IntPtr.Zero, false, false, null);
            hRecvEvent[1] = hMsgQ;

            recvThread = new Thread(new ThreadStart(this.RecvThreadPorc));
            recvThread.Start();


            for (int i = 0; i < hSendEvent.Length; ++i)
            {
                hSendEvent[i] = Win32.CreateEvent(IntPtr.Zero, false, false, null);
            }
            smsQueue = new Queue<Sms>();

            sendThread = new Thread(new ThreadStart(this.SendThreadPorc));
            sendThread.Start();   


            isOpened = true;
            return true;

        }

        public void Close()
        {
            try
            {
                Win32.EventModify(hSendEvent[0], Win32.EVENT_SET);
                if (!sendThread.Join(1000))
                {
                    sendThread.Abort();
                }

                for (int i = 0; i < hSendEvent.Length; ++i)
                {
                    Win32.CloseHandle(hSendEvent[i]);
                }


                Device.SMS_UnRegisterNotification(hNotify);
                Win32.EventModify(hRecvEvent[0], Win32.EVENT_SET);
                if (!recvThread.Join(1000))
                {
                    recvThread.Abort();
                }

                Win32.CloseHandle(hRecvEvent[0]);
                Win32.CloseMsgQueue(hRecvEvent[1]);


                if (needClose)
                {
                    Device.SMS_Close();
                }

                isOpened = false;

            }
            catch (Exception ex)
            {
            }
        }

        private void SendThreadPorc()
        {
            ushort id = 0;

            while (true)
            {
                uint ret = Win32.WaitForMultipleObjects(2, hSendEvent, false, Win32.INFINITE);
                switch (ret)
                {
                    case 0:
                        return;
                    case 1:
                        if (smsQueue.Count > 0)
                        {
                            Sms sms = smsQueue.Peek();  

                            bool result = false;

                            int limit = 70; //中文短信每条最多70
                            int limit2 = 67;    //长中文短信每条最多67

                            int wcsLen = sms.Content.Length;
                            int mbsLen = Encoding.ASCII.GetByteCount(sms.Content);
                            if (wcsLen == mbsLen) //全英文
                            {
                                limit = 160;
                                limit2 = 153;
                            }

                            if (wcsLen > limit)
                            {
                                id++;
                                if (id > 0xff)
                                {
                                    id = 0;
                                }

                                int nTotal = wcsLen / limit2;   //总条数
						        if(wcsLen % limit2 > 0)
						        {
							        nTotal++;
						        }

                                result = true;
                             

                                for (int i = 0; i < nTotal; ++i)
                                {
                                    string str = sms.Content.Substring(i * limit2, (i + 1 == nTotal) ? (wcsLen % limit2) : limit2 );
                                    if (!Device.SMS_SendSMSEx(sms.PhoneNum, "", str, id, (byte)nTotal, (byte)(i + 1)))
                                    {
                                        result = false;
                                        break;
                                    }
                                }

                            }
                            else
                            {
                                result = Device.SMS_SendSMSEx(sms.PhoneNum, "", sms.Content, 0, 0, 0);
                            }

                            EventHandler<SmsSendResultEventArgs> temp = SmsSent;
                            if (temp != null)
                            {
                                temp(this, new SmsSendResultEventArgs(result));
                            }                          

                            //发送下一条
                            smsQueue.Dequeue();
                            Win32.EventModify(hSendEvent[1], Win32.EVENT_SET);
                        }

                        break;
                }
            }
        }


        private void RecvThreadPorc()
        {
            while (true)
            {
                uint evt = Win32.WaitForMultipleObjects(2, hRecvEvent, false, Win32.INFINITE);
                switch (evt)
                {
                    case 0://return thread
                        return;
                    case 1://disable network 
                        uint bytesRead;
                        uint flags;

                        byte[] buf = new byte[2];

                        if (Win32.ReadMsgQueue(hRecvEvent[1], buf, 2, out bytesRead, Win32.INFINITE, out flags))
                        {
                            switch ((Device.SmsNotifyType)buf[0])
                            {
                                case Device.SmsNotifyType.NewSms:
                                    EventHandler<SmsReceivedEventArgs> temp = SmsReceived;
                                    if (temp != null)
                                    {
                                        temp(this, new SmsReceivedEventArgs(buf[1]));
                                    }
                                    break;
                                case Device.SmsNotifyType.Rssi:
                                    break;
                                case Device.SmsNotifyType.RegistrationState:
                                    break;
	                            default:
                                    break;
                            }

                        }
                        break;
                }
            }
        }


        public bool SendSMS(string szRecipient, string szMsg)
        {
            if (isOpened)
            {

                smsQueue.Enqueue(new Sms(szRecipient, szMsg));
                Win32.EventModify(hSendEvent[1], Win32.EVENT_SET);

                return true;
            }

            return false;
        }


        public bool ReadSMS(int iIndex, StringBuilder szRecipient, uint dwRecpLen, StringBuilder szMsg, uint dwMsgLen, StringBuilder szTime, uint dwTimeLen, ref ushort wid, ref byte byTotalNum, ref byte byCurNum)
        {
            return Device.SMS_ReadSMSEx(iIndex, szRecipient, dwRecpLen, szMsg, dwMsgLen, szTime, dwTimeLen, ref wid,ref byTotalNum, ref byCurNum);
        }


        public bool DeleteSMS(int iIndex)
        {
            return Device.SMS_DeleteSMS(iIndex);
        }


    }
 
}
